/*
  Esempio di utilizzo di Node e Net per un piccolo programma di Chat
*/

import java.io.* ;
import java.awt.* ;
import java.util.Enumeration ;
import java.util.Vector ;

import NetNode.* ;

public class ChatNode extends Node {
  protected ChatFrame frame ;
  protected static final int PERSONAL = 1 ;
    
  public ChatNode (String name, String host, int port)
        throws IOException {
    super(name, host, port) ;
    frame = newFrame(name, this);
  }
  
  protected ChatFrame newFrame(String title, ChatNode cc) { 
    return new ChatFrame(title,cc) ; 
  }
  
  public void SendString( String s ) throws IOException {
    String[] names = frame.GetSelectedNames() ;
    if ( names.length > 0 ) {
        for ( int i = 0 ; i < names.length ; i++ )
            sendMessage( names[i], ChatNode.PERSONAL, s ) ;
    } else
        broadcast( s, true ) ; 
  }

  public void startNode() {
      frame.show ();      
      getAllNodesNames() ;
  }
  
  public void handleMessage( NodeMessage message ) {
    switch ( message.OpCode ) {
        case Net.NEWNODE_NOTIFY :
            frame.AddText( "--- " + (String)message.Content + 
                " e' entrato in chat" ) ;
            frame.AddName( (String)message.Content ) ;
            break ;
        case Net.NODEREMOVED_NOTIFY :
            frame.AddText( "--- " + (String)message.Content + 
                " ha lasciato la chat" ) ;
            frame.RemoveName( (String)message.Content ) ;
            break ;
        case Net.GETALLNODES :
            frame.FillList( (Vector) message.Content ) ;
            break ;
        default :
            frame.AddText( 
                ( message.Source.equals( nodeName ) ? "> " : 
                "(" + message.Source + 
                ( message.OpCode == ChatNode.PERSONAL ? 
                " PERSONAL " : "" ) +") " ) + 
                (String) message.Content ) ;
    }
  }
  
  public static void main (String args[]) throws IOException {
    if (args.length != 3)
      throw new RuntimeException ("Sintassi: ChatNode <nome> <host> <port>");
    
    (new ChatNode ("Ch " + args[0], args[1], 
        Integer.parseInt(args[2]))).start();
  }
}

class ChatFrame extends Frame {
    ChatNode chatNode ;
    protected TextArea output = new TextArea();
    protected TextField input = new TextField();
    protected List namesList = new List() ;
    
    public ChatFrame( String title, ChatNode cc ) { 
        super(title) ;
        chatNode = cc ;
        setLayout(new BorderLayout ());
        add("Center", output);
        output.setEditable(false);
        add("South", input);
        namesList.setMultipleMode( true ) ;
        add("East", namesList ) ;
        pack ();        
        input.requestFocus();   
    }
    
    public boolean handleEvent (Event e) {
        if ((e.target == input) && (e.id == Event.ACTION_EVENT)) {
            try {
                chatNode.SendString((String) e.arg);
            } catch (IOException ex) {
                ex.printStackTrace();
                chatNode.stop ();
            }
            input.setText ("");
            return true;    
        } else if ((e.target == this) && (e.id == Event.WINDOW_DESTROY)) {
            chatNode.stop ();
            hide ();
            return true;
        }
        return super.handleEvent (e);
    }
    
    public void AddText( String s ) { output.appendText( s + "\n" ) ; }
    public void AddName( String s ) { namesList.add( s ) ; }
    public void RemoveName( String s ) { namesList.remove( s ) ; }
    public String[] GetSelectedNames() { return namesList.getSelectedItems() ; }
    public void FillList( Vector v ) {
        synchronized ( namesList ) {
            namesList.removeAll() ;
            Enumeration en = v.elements() ;
            while( en.hasMoreElements() ) {
                namesList.add( (String)en.nextElement() ) ;
            }
        }
    }
}
