package NetNode ;

import java.io.* ;
import java.net.* ;
import java.util.Hashtable ;
import java.util.Enumeration ;
import java.util.Vector ;

public class Net extends Thread {
        // costanti
        public static final int NEWNODE_NOTIFY = -1 ;
        public static final int NODEREMOVED_NOTIFY = -2 ;
        public static final int BROADCAST = -3 ;
        public static final int BROADCASTBUTNOTME = -4 ;
        public static final int NOSUCHNODE_ERROR = -5 ;
        public static final int GETALLNODES = -6 ;
		
	protected final static int defaultPort = 9999 ;
	protected ServerSocket serversocket ;
        // nodi della rete : ( nome nodo, nodehandler )
	protected Hashtable nodes = new Hashtable() ;

	public Net() throws IOException {
	        super( "Net su porta " + Net.defaultPort ) ;
			serversocket = new ServerSocket( Net.defaultPort ) ;
	}

	public Net( int port ) throws IOException {
	        super( "Net su porta " + port ) ;
		serversocket = new ServerSocket( port ) ;			
	}

	public void run() {
		Socket socket ;
		NodeHandler nhandler ;
		try {
			while( true ) {
				socket = serversocket.accept() ;
        	            	Print( "Nuovo nodo da " + 
					socket.getInetAddress().getHostAddress() +
					    ":" + socket.getPort() ) ;
					nhandler = new NodeHandler( this, socket.getInputStream(),
					    socket.getOutputStream(), nodes ) ;
					nhandler.start() ;
			}
		} catch ( IOException e ) {
                	e.printStackTrace() ;
		}
	}

        public boolean isNode( String l ) {
		synchronized ( nodes ) {
				return nodes.containsKey( l.toString() ) ;
		}
	}	

        public void addNode( String l, NodeHandler nhandler ) {
		synchronized ( nodes ) {
			notifyNodes( "Net", nhandler, Net.NEWNODE_NOTIFY, 
			    nhandler.getHandledNode(), false ) ;
			nodes.put( l.toString(), nhandler ) ;				
		}
		PrintNewNode( l ) ;
	}

        public void removeNode( String l ) {
		synchronized ( nodes ) {
			NodeHandler nhandler = (NodeHandler)nodes.remove( l.toString() ); 
			notifyNodes( "Net", nhandler, 
			    Net.NODEREMOVED_NOTIFY, nhandler.getHandledNode(), false ) ;
		}
		PrintRemovedNode( l ) ;
	}
		
	public void notifyNodes( String source, NodeHandler nhandler, int notify, 
		        Object content, boolean meToo ) {
		NodeHandler n ;
		Enumeration en = nodes.elements() ;
		while ( en.hasMoreElements() ) {
			n = ( NodeHandler )en.nextElement() ;
			if ( n != nhandler || meToo ) {
				n.Enqueue( new NodeMessage( source, n.getHandledNode(),
					Thread.currentThread().getName(), 
				        notify, content ) ) ;
			}
		}
	}

        public NodeHandler getNodeHandler( String l ) {
		return ( NodeHandler ) nodes.get( l.toString() ) ;
	}
		
	public Vector getAllNodesNames() {
		Vector allNodes = new Vector() ;
		Enumeration en = nodes.elements() ;
		while ( en.hasMoreElements() )
			allNodes.addElement( 
		        	((NodeHandler)en.nextElement()).getHandledNode() ) ;
		return allNodes ;
	}
		
	protected void Print( String s ) {
		System.out.println( s ) ;
	}

        protected void PrintNewNode( String l ) {
		Print( "Nuovo nodo alla localita' : " + l ) ;
	}

        protected void PrintRemovedNode( String l ) {
		Print( "Tolto nodo alla localita' : " + l ) ;
	}

	public static void main( String args[] ) throws IOException {
		if ( args.length > 1 )
			throw new IOException( "Sintassi : Net <port>" ) ;
		int port = Net.defaultPort ;
		if ( args.length == 1 )
			port = Integer.parseInt( args[ 0 ] ) ;
        	System.out.println( "Net sull porta " + port ) ;
		Net net = new Net( port ) ;
		net.start() ;
	}
}
			
