package NetNode ;

import java.io.* ;
import java.net.* ;
import java.util.Vector ;
import java.util.Hashtable ;
import java.util.Enumeration ;

public class Node extends Thread {
  
  protected String nodeName ;
  protected Queue outMessageQueue ; // coda dei messaggi in uscita
  protected Queue inMessageQueue ; // coda dei messaggi in entrata
  protected InputStream iStream ; // per la comunicazione in rete
  protected OutputStream oStream ;

  protected String netHostName ;
  protected int netPort ;

  public Node( String Name, String NetHostName, int NetPort )
      throws IOException {
    super( "Node - " + Name ) ;   
      nodeName = Name ;
    netHostName = NetHostName ;
    netPort = NetPort ;
    
    connect( netHostName, netPort ) ;
    login( nodeName ) ;
    outMessageQueue = new Queue() ;
    inMessageQueue = new Queue() ;
  }
  
  protected void connect( String host, int port ) throws IOException {
    System.out.println( "Connessione con " + host + ":" + port + " ...") ;
    Socket s = new Socket( host, port ) ;
    System.out.println( "Socket creata..." ) ;
    iStream = s.getInputStream() ;
    oStream = s.getOutputStream() ;
    System.out.println( "Connessione stabilita." ) ;    
  }

  protected void login( String nodeName ) throws IOException {
    try {
      System.out.println( "Login " + nodeName + " ..." ) ;
      ObjectOutputStream o = new ObjectOutputStream( oStream ) ;
      o.writeObject( nodeName ) ;     
      ObjectInputStream i = new ObjectInputStream( iStream ) ;
      Boolean registered = (Boolean)i.readObject() ;
      if ( ! registered.booleanValue() )
        throw new IOException( "Nome " + nodeName + " gi in uso" ) ;
      System.out.println( "Login con successo!" ) ;
    } catch ( ClassNotFoundException ce ) {
      ce.printStackTrace() ;
    } catch ( IOException ex ) {
      try {
        ex.printStackTrace() ;
        oStream.close() ;
      } catch ( IOException ex2 ) {
        ex.printStackTrace() ;
      }
      // rilancia la prima eccezione
      throw ex ;
    }
  }

  public void run() {
    MessageDeliverer deliverer ;
    MessageReceiver receiver ;

    try {
      deliverer = new MessageDeliverer(
        this, outMessageQueue, new ObjectOutputStream( oStream ) ) ;
      receiver = new MessageReceiver(
        this, inMessageQueue, new ObjectInputStream( iStream ) ) ;
    } catch ( IOException ioe ) {
      System.err.println( ioe ) ;
      return ;
    }

    try {
      deliverer.start() ;
      receiver.start() ;  
      listen() ;
    } catch ( Exception e ) {
      System.err.println( e ) ;
    } finally {
      deliverer.stop() ;
      receiver.stop() ;
      closeConnection() ;
      System.exit(0) ;
    }
  }

  protected void closeConnection() {
    System.out.println( "Chiusura della connessione..." ) ;
    try {
      oStream.close() ;
    } catch ( IOException e ) {
      System.err.println( e ) ;
    }
  }

  protected void listen() throws IOException {
    startNode() ;
    Print( "In ascolto di messaggi..." ) ;

    NodeMessage message ;
    while ( true ) {
      message = ( NodeMessage )inMessageQueue.remove() ;
      PrintMessage( "Ricevuto: " + message ) ;        
      handleMessage( message ) ;
    }
  }

  protected void handleMessage( NodeMessage m ) {
    Print( "handleMessage : " + m.toString() ) ;
  }
    
  protected void startNode() {
    Print( "Nodo attivo..." ) ;        
  }
    
  public void sendMessage( String dest, int opCode, Object content ) {
    sendMessage( nodeName, dest, Thread.currentThread().getName(),
      opCode, content ) ;
  }
    
  public void sendMessage( String source, String dest,
      String processname, int opcode, Object content ) {
    NodeMessage mesg = new NodeMessage( source, dest, processname, opcode, content ) ;
    PrintMessage( "Sending " + mesg ) ;
    outMessageQueue.add( mesg ) ;
  }
    
  public void broadcast( Object content, boolean meToo ) {
    if ( meToo )
      sendMessage( null, Net.BROADCAST, content ) ;
    else
      sendMessage( null, Net.BROADCASTBUTNOTME, content ) ;
  }
  
  public void getAllNodesNames() {
    sendMessage( null, Net.GETALLNODES, null ) ;
  }

  synchronized protected void Print( String s ) {
    System.out.println( s ) ;
  }

  synchronized protected void PrintMessage( String s ) {
    System.out.println( s ) ;
  }

  synchronized public void PrintQueue() {
    Print( "Coda messaggi : " ) ;
    Print( "" + outMessageQueue ) ;
  }
    
  public String toString() {
    return ("Node : " + nodeName) ;
  }

  public static void main( String args[] ) throws IOException {
    if ( args.length != 3 )
      throw new IOException( "Syntax : Node <nodeName> <hostname> <port>" ) ;

    Node n = new Node( args[0], args[1], Integer.parseInt( args[2] ) ) ;
    n.start() ;   
  }        
}