package NetNode ;

import java.io.* ;
import java.net.* ;
import java.util.Hashtable ;

public class NodeHandler extends Thread {
	static private int handlerNumber = 0 ;
	protected Queue messageQueue ; // coda dei messaggi
	// streams per la comunicazione in rete
	protected InputStream iStream ; 
	protected OutputStream oStream ;
	protected Net net ; // la rete a cui appartiene
	protected Hashtable nodes ; // in nodi della rete
	protected String handledNode ;

	public NodeHandler( Net net, InputStream i, OutputStream o, Hashtable n ) throws IOException {
		super( "NodeHandler - " + NextHandlerNumber() ) ;
		this.net = net ;
		nodes = n ;
		iStream = i ;
		oStream = o ;
	}

	static private synchronized int NextHandlerNumber() {
		return handlerNumber++ ;
	}

	public void run() {
		try {
			ObjectInputStream iObjStream = new ObjectInputStream( iStream ) ;
			try {
				String name = ( String ) iObjStream.readObject() ;
				register( name ) ;
			} catch ( ClassNotFoundException nfe ) {
				System.err.println( nfe ) ;
			}
        	} catch ( IOException e ) {
		} finally {
			try {
				oStream.close() ;
			} catch ( IOException e1 ) {
                		e1.printStackTrace() ;
			}
		}
	}

	protected void register( String name ) throws IOException {
		messageQueue = new Queue() ;
		boolean registered = true ;
		
		synchronized( nodes ) {
    		if ( net.isNode( name ) ) {
    			registered = false ;
    		} else {
    			handledNode = name.toString() ;
    			net.addNode( name, this ) ;			
    		}
		}

		try {
			ObjectOutputStream O = new ObjectOutputStream( oStream ) ;
			O.writeObject( new Boolean( registered ) ) ;			
			if ( registered ) {				
				execute() ;				
			}
		} finally {
			if ( registered ) {
				net.removeNode( name ) ;				
			}
		}
	}

	protected void execute() throws IOException {
		MessageDeliverer deliverer = new MessageDeliverer(
			this, messageQueue, new ObjectOutputStream( oStream ) ) ;

		try {
			deliverer.start() ;
			listen() ;
		} finally {
			deliverer.stop() ;
		}
	}
	
	protected void listen() throws IOException {
		ObjectInputStream iObjStream = new ObjectInputStream( iStream ) ;
		NodeMessage message ;

		while( true ) {
		    try {
			    message = ( NodeMessage ) iObjStream.readObject() ;
			    handleMessage( message ) ;
			} catch ( ClassNotFoundException e ) {
			    e.printStackTrace() ;
			}
		}
	}

	protected void handleMessage( NodeMessage message ) throws IOException {
		NodeHandler nHandler ;
		String name ;
		
		switch ( message.OpCode ) {
			case Net.BROADCAST :
				net.notifyNodes( handledNode, this, message.OpCode, 
    				        message.Content, true ) ;
    				break ;
    			case Net.BROADCASTBUTNOTME :
    				net.notifyNodes( handledNode, this, message.OpCode, 
					message.Content, false ) ;
    				break ;
    			case Net.GETALLNODES :
    				message.Content = net.getAllNodesNames() ;
				Enqueue( message ) ;
                        	break ;
                    	default :    				
        			name = message.Dest ;
                        	nHandler = net.getNodeHandler( name ) ;                        
				if ( nHandler != null )
            		        	nHandler.Enqueue( message ) ;
                        	else {
                            		Print( name + " non esiste!" ) ;
        			        message.OpCode = Net.NOSUCHNODE_ERROR ;			        
        			        Enqueue( message ) ;
                		}
        		}		
	}

	public void Enqueue( NodeMessage m ) {
		messageQueue.add( m ) ;
	}
	
	public String getHandledNode() { return handledNode ; }

	protected void Print( String s ) {
		System.out.println( handledNode + ": " + s ) ;
	}
}
